package com.rehome.jpahefengweather.service;


import com.rehome.jpahefengweather.entity.WztfStorm;
import com.rehome.jpahefengweather.entity.WztfStormInfo;

import java.util.List;

public interface WztfStormService {
    //根据id查询台风
    WztfStorm findByTfbh(String tfbh);
    //根据年份查询台风列表
    List<WztfStorm> findByYear(String year);
    /**
     * 保存台风数据
     */
    void saveWztfStorm(WztfStorm wztfStorm);

    WztfStormInfo findWztfStormInfoByTfbh(String tfbh);
    /**
     * 保存台风路径数据
     */
    void saveWztfStormInfo(WztfStormInfo wztfStormInfo);
}
